'use strict';

const startTimeKey = Symbol('startTime');

const SET_COOKIE = 'set-cookie';
const PROXY_AUTHENTICATE = 'proxy-authenticate';

/**
 * response serializer
 *
 * removes set-cookie and proxy-authenticate headers
 * @param {object} [res]
 * @returns {object}
 */
function resSerializer (res) {
  if (typeof res !== 'object' || !res) return

  const {
    [SET_COOKIE]: _1,
    [PROXY_AUTHENTICATE]: _2,
    ...headers
  } = res.getHeaders ? res.getHeaders() : res._headers || {};

  const logRes = {
    statusCode: res.statusCode
  };
  if (Object.keys(headers).length) {
    logRes.headers = headers;
  }
  if (res[startTimeKey]) {
    logRes.ms = Date.now() - res[startTimeKey];
  }

  return logRes
}

exports.resSerializer = resSerializer;
exports.startTimeKey = startTimeKey;
