'use strict';

var utils = require('./utils.cjs');

const LEVELS = [utils.LOG, utils.TRACE, utils.DEBUG, utils.INFO, utils.WARN, utils.ERROR];

let wrapped = null;

const unwrap = () => {
  if (wrapped) {
    LEVELS.forEach(level => {
      console[level] = wrapped[level];
    });
    wrapped = null;
  }
};

/**
 * wrap console logging functions like
 * console.log, console.info, console.warn, console.error
 * @return unwrap function
 */
function wrapConsole (log, { level4log = 'LOG' } = {}) {
  if (wrapped) return unwrap
  wrapped = {};

  const _level4log = utils.adjustLevel(level4log, 'LOG').toLowerCase();

  const render = level => (...args) => {
    log[level](...args);
  };

  LEVELS.map(l => l.toLowerCase()).forEach(level => {
    wrapped[level] = console[level];
    const renderLevel = level === 'log'
      ? _level4log
      : level;
    console[level] = render(renderLevel);
  });

  return unwrap
}

exports.wrapConsole = wrapConsole;
